@@ -134,7 +134,7 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
 
     async def before_update(
         self, user: UserModel, obj: Model, input: UpdateForm, request: Request
-    ) -> Model:
+    ) -> Optional[Model]:
         return obj
 
     async def after_update(
@@ -227,12 +227,9 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
                         raw: Dict[str, Any] = input.dict(exclude_unset=True)
                         obj = self.model(**raw)
 
-                        obj_rtn = await self.before_create(
+                        obj = await self.before_create(
                             current_user, obj, input, request
                         )
-                        if obj_rtn is not obj:
-                            obj = obj_rtn
-
                         await obj.save()
                         if prefetch:
                             await obj.fetch_related(*prefetch)
@@ -240,7 +237,7 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
                         obj_rtn = await self.after_create(
                             current_user, obj, input, request
                         )
-                        if obj_rtn and obj_rtn is not obj:
+                        if obj_rtn:
                             obj = obj_rtn
 
                         if isinstance(obj, PydanticBaseModel):
@@ -422,7 +419,6 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
             ) -> Any:
                 async with in_transaction(self.connection_name):
                     rtns: List[Any] = []
-                    raw = input.dict(exclude_unset=True)
                     for id in ids.split(","):
                         id = id.strip()
                         id = await self.translate_id(current_user, id, request)
@@ -433,34 +429,32 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
                             ResourceMethod.put,
                         ).get()
 
-                        obj_rtn = await self.before_update(
-                            current_user, obj, input, request
-                        )
-                        if obj_rtn is not obj:
-                            obj = obj_rtn
+                        r = await self.before_update(current_user, obj, input, request)
+                        if r:
+                            obj = r
+                            raw = input.dict(exclude_unset=True)
+                            updated_at = raw.pop("updated_at", None)
+                            if updated_at:
+                                # 防止老数据修改
+                                if isinstance(updated_at, str):
+                                    updated_at = json.parse_iso_datetime(updated_at)
+                                if isinstance(updated_at, datetime):
+                                    if obj.updated_at > updated_at:
+                                        raise HTTPPreconditionRequiredError
 
-                        updated_at = raw.pop("updated_at", None)
-                        if updated_at:
-                            # 防止老数据修改
-                            if isinstance(updated_at, str):
-                                updated_at = json.parse_iso_datetime(updated_at)
-                            if isinstance(updated_at, datetime):
-                                if obj.updated_at > updated_at:
-                                    raise HTTPPreconditionRequiredError
+                            update_fields = raw.keys()
+                            if update_fields:
+                                obj.update(raw)
+                                await obj.save(update_fields=update_fields)
 
-                        update_fields = raw.keys()
-                        if update_fields:
-                            obj.update(raw)
-                            await obj.save(update_fields=update_fields)
+                            if prefetch:
+                                await obj.fetch_related(*prefetch)
 
-                        if prefetch:
-                            await obj.fetch_related(*prefetch)
-
-                        obj_rtn = await self.after_update(
-                            current_user, obj, input, request
-                        )
-                        if obj_rtn and obj_rtn is not obj:
-                            obj = obj_rtn
+                            obj_rtn = await self.after_update(
+                                current_user, obj, input, request
+                            )
+                            if obj_rtn:
+                                obj = obj_rtn
 
                         rtns.append(
                             obj
@@ -493,9 +487,7 @@ class API(Generic[UserModel, Model, CreateForm, UpdateForm]):
                             self.model.filter(id=id),
                             ResourceMethod.delete,
                         ).get()
-                        obj_rtn = await self.before_delete(current_user, obj, request)
-                        if obj_rtn is not obj:
-                            obj = obj_rtn
+                        obj = await self.before_delete(current_user, obj, request)
                         await obj.delete()
                         rs.append(DeleteResponse(id=id))
                     return len(rs) > 1 and rs or rs[0]
